from django.conf.urls.defaults import *
from billing import get_integration

pay_pal = get_integration("pay_pal")
fps = get_integration("amazon_fps")
braintree = get_integration("braintree_payments")

urlpatterns = patterns('',
      ('^paypal-ipn-url/', include(pay_pal.urls)),
      ('^fps/', include(fps.urls)),
      ('^braintree/', include(braintree.urls)),
)



from django.test import TestCase
from billing import get_integration
from django.template import Template, Context
from django.conf import settings

class AmazonFPSTestCase(TestCase):
    urls = "billing.tests.test_urls"

    def setUp(self):
        self.fps = get_integration("amazon_fps")
        fields = {
            "callerReference": "100",
            "paymentReason": "Digital Download",
            "pipelineName": "SingleUse",
            "transactionAmount": 30,
            "returnURL": "http://localhost/fps/fps-return-url/",
            }
        self.fps.add_fields(fields)

    def testLinkGen(self):
        tmpl = Template("{% load amazon_fps from amazon_fps_tags %}{% amazon_fps obj %}")
        link = tmpl.render(Context({"obj": self.fps}))
        pregen_link = """<a href="https://authorize.payments-sandbox.amazon.com/cobranded-ui/actions/start?callerKey=%(aws_access_key)s&callerReference=100&paymentReason=Digital%%20Download&pipelineName=SingleUse&returnURL=http%%3A%%2F%%2Flocalhost%%2Ffps%%2Ffps-return-url%%2F&signature=oSnkew7oCBPVk0IVZAjO87Ogsp4EO7jRlELaFwtqWzY%%3D&signatureMethod=HmacSHA256&signatureVersion=2&transactionAmount=30"><img src="http://g-ecx.images-amazon.com/images/G/01/cba/b/p3.gif" alt="Amazon Payments" /></a>""" %({"aws_access_key": settings.MERCHANT_SETTINGS['amazon_fps']['AWS_ACCESS_KEY']})
        self.assertEquals(pregen_link, link.strip())



from django import forms
import decimal, datetime

curr_year = datetime.datetime.now().year
month_choices = ((ii, ii) for ii in range(1, 13))
year_choices = ((ii, ii) for ii in range(curr_year, curr_year + 7))

class StripeForm(forms.Form):
    # Small value to prevent non-zero values. Might need a relook
    amount = forms.DecimalField(min_value=decimal.Decimal('0.001'))
    credit_card_number = forms.CharField(max_length=16)
    credit_card_cvc = forms.CharField(max_length=4)
    credit_card_expiration_month = forms.CharField(max_length=2, widget=forms.Select(choices=month_choices))
    credit_card_expiration_year = forms.CharField(max_length=4, widget=forms.Select(choices=year_choices))



from django.test import TestCase
from billing import get_gateway, CreditCard
from billing.signals import *
from billing.gateway import CardNotSupported, InvalidData
from billing.utils.credit_card import Visa
import braintree

class BraintreePaymentsGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("braintree_payments")
        self.merchant.test_mode = True
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2011, 
                                      number="4111111111111111", 
                                      verification_value="100")

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported, 
                          lambda : self.merchant.purchase(1000, self.credit_card))

    def testCardType(self):
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")

    def testFailedPurchase(self):
        resp = self.merchant.purchase(2001, self.credit_card)
        self.assertEquals(resp["status"], "FAILURE")

    def testDeclinedPurchase(self):
        resp = self.merchant.purchase(2900, self.credit_card)
        self.assertEquals(resp["status"], "FAILURE")

    def testPaymentSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_successful.connect(receive)

        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(received_signals, [transaction_was_successful])

    def testPaymentUnSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_unsuccessful.connect(receive)

        resp = self.merchant.purchase(2000, self.credit_card)
        self.assertEquals(received_signals, [transaction_was_unsuccessful])

    def testCreditCardExpired(self):
        credit_card = CreditCard(first_name="Test", last_name="User",
                                 month=10, year=2011, 
                                 number="4000111111111115", 
                                 verification_value="100")
        resp = self.merchant.purchase(2004, credit_card)
        self.assertNotEquals(resp["status"], "SUCCESS")

    def testAuthorizeAndCapture(self):
        resp = self.merchant.authorize(100, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.capture(50, resp["response"].transaction.id)
        self.assertEquals(response["status"], "SUCCESS")

    # Need a way to test this. Requires delaying the status to either
    # "settled" or "settling"
    # def testAuthorizeAndRefund(self):
    #     resp = self.merchant.purchase(100, self.credit_card)
    #     self.assertEquals(resp["status"], "SUCCESS")
    #     response = self.merchant.credit(50, resp["response"].transaction.id)
    #     self.assertEquals(response["status"], "SUCCESS")

    def testAuthorizeAndVoid(self):
        resp = self.merchant.authorize(100, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.void(resp["response"].transaction.id)
        self.assertEquals(response["status"], "SUCCESS")

    def testStoreMissingCustomer(self):
        self.assertRaises(InvalidData, 
                          lambda : self.merchant.store(self.credit_card, {}))

    def testStoreWithoutBillingAddress(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            }
        resp = self.merchant.store(self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        self.assertEquals(resp["response"].customer.credit_cards[0].expiration_date, 
                          "%s/%s" %(self.credit_card.month,
                                    self.credit_card.year))
        self.assertTrue(getattr(resp["response"].customer.credit_cards[0], "customer_id"))
        self.assertTrue(getattr(resp["response"].customer.credit_cards[0], "token"))

    def testStoreWithBillingAddress(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            "billing_address": {
                "name": "Johnny Doe",
                "company": "",
                "email": "johnny.doe@example.com",
                "address1": "Street #1",
                "address2": "House #2",
                "city": "Timbuktu",
                "country": "United States of America",
                "zip": "110011"
                }
            }
        resp = self.merchant.store(self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        self.assertTrue(getattr(resp["response"].customer.credit_cards[0], "billing_address"))
        # The tests below don't seem to work.
        # billing_address = resp["response"].customer.credit_cards[0].billing_address
        # self.assertEquals(billing_address.country_code_alpha2, "US")
        # self.assertEquals(billing_address.postal_code, "110011")
        # self.assertEquals(billing_address.street_address, "Street #1")
        # self.assertEquals(billing_address.extended_address, "House #2")
        # self.assertEquals(billing_address.city, "Timbuktu")

    def testUnstore(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            }
        resp = self.merchant.store(self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.unstore(resp["response"].customer.credit_cards[0].token)
        self.assertEquals(response["status"], "SUCCESS")

    # The below tests require 'test_plan' to be created in the sandbox 
    # console panel. This cannot be created by API at the moment
    def testRecurring1(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            "recurring": {
                "plan_id": "test_plan"
                },
            }
        resp = self.merchant.recurring(10, self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        subscription = resp["response"].subscription
        self.assertEquals(subscription.status,
                          braintree.Subscription.Status.Active)

    def testRecurring2(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            "recurring": {
                "plan_id": "test_plan",
                "price": 15
                },
            }
        resp = self.merchant.recurring(10, self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        subscription = resp["response"].subscription
        self.assertEquals(subscription.price, 15)

    def testRecurring3(self):
        options = {
            "customer": {
                "name": "John Doe",
                "email": "john.doe@example.com",
                },
            "recurring": {
                "plan_id": "test_plan",
                "trial_duration": 2,
                "trial_duration_unit": "month",
                "number_of_billing_cycles": 12,
                },
            }
        resp = self.merchant.recurring(10, self.credit_card, options = options)
        self.assertEquals(resp["status"], "SUCCESS")
        subscription = resp["response"].subscription
        self.assertEquals(subscription.number_of_billing_cycles, 12)



from billing import Integration, IntegrationNotConfigured
from django.conf import settings
from paypal.standard.conf import POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT
from django.conf.urls.defaults import patterns, include
from paypal.standard.ipn.signals import payment_was_flagged, payment_was_successful
from billing.signals import transaction_was_successful, transaction_was_unsuccessful

class PayPalIntegration(Integration):
    display_name = "PayPal IPN"

    def __init__(self):
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("pay_pal"):
            raise IntegrationNotConfigured("The '%s' integration is not correctly "
                                       "configured." % self.display_name)
        pay_pal_settings = merchant_settings["pay_pal"]
        # Required Fields. Just a template for the user
        self.fields = {"business": pay_pal_settings['RECEIVER_EMAIL'],
                       "item_name": "",
                       "invoice": "",
                       "notify_url": "",
                       "return_url": "",
                       "cancel_return": "",
                       "amount": 0,
                       }

    @property
    def service_url(self):
        if self.test_mode:
            return SANDBOX_POSTBACK_ENDPOINT
        return POSTBACK_ENDPOINT

    def get_urls(self):
        urlpatterns = patterns('',
           (r'^', include('paypal.standard.ipn.urls')),
            )
        return urlpatterns

def unsuccessful_txn_handler(sender, **kwargs):
    transaction_was_unsuccessful.send(sender=sender.__class__,
                                      type="purchase",
                                      response=sender)

def successful_txn_handler(sender, **kwargs):
    transaction_was_successful.send(sender=sender.__class__,
                                    type="purchase",
                                    response=sender)

payment_was_flagged.connect(unsuccessful_txn_handler)
payment_was_successful.connect(successful_txn_handler)



from billing import Gateway, GatewayNotConfigured
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from billing.utils.credit_card import InvalidCard, Visa, MasterCard, \
     AmericanExpress, Discover, CreditCard
import samurai
import samurai.config as samurai_config
from samurai.payment_method import PaymentMethod
from samurai.processor import Processor
from samurai.transaction import Transaction
from django.conf import settings


class SamuraiGateway(Gateway):
    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    supported_countries = ['US']
    default_currency = "USD"
    homepage_url = "https://samurai.feefighters.com/"
    display_name = "Samurai"

    def __init__(self):
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("samurai"):
            raise GatewayNotConfigured("The '%s' gateway is not correctly "
                                       "configured." % self.display_name)
        samurai_settings = merchant_settings["samurai"]
        samurai_config.merchant_key = samurai_settings['MERCHANT_KEY']
        samurai_config.merchant_password = samurai_settings['MERCHANT_PASSWORD']
        samurai_config.processor_token = samurai_settings['PROCESSOR_TOKEN']
        self.samurai = samurai

    def purchase(self, money, credit_card):
        # Cases where token is directly sent for e.g. from Samurai.js
        payment_method_token = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            pm = PaymentMethod.create(credit_card.number, credit_card.verification_value,
                                      credit_card.month, credit_card.year)
            payment_method_token = pm.payment_method_token
        response = Processor.purchase(payment_method_token, money)
        if response.errors:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="purchase",
                                              response=response)
            return {'status': 'FAILURE', 'response': response}
        transaction_was_successful.send(sender=self, 
                                        type="purchase",
                                        response=response)
        return {'status': 'SUCCESS', 'response': response}

    def authorize(self, money, credit_card, options=None):
        payment_method_token = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            pm = PaymentMethod.create(credit_card.number, credit_card.verification_value, 
                                      credit_card.month, credit_card.year)
            payment_method_token = pm.payment_method_token
        response = Processor.authorize(payment_method_token, money)
        if response.errors:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="authorize",
                                              response=response)
            return {'status': 'FAILURE', 'response': response}
        transaction_was_successful.send(sender=self, 
                                        type="authorize",
                                        response=response)
        return {'status': 'SUCCESS', 'response': response}

    def capture(self, money, identification, options=None):
        trans = Transaction.find(identification)
        if not trans.errors:
            new_trans = trans.capture(money)
            transaction_was_successful.send(sender=self, 
                                            type="capture",
                                            response=trans)
            return{'status': "SUCCESS", "response": new_trans}
        transaction_was_unsuccessful.send(sender=self, 
                                          type="capture",
                                          response=trans)
        return{"status": "FAILURE", "response": trans}

    def void(self, identification, options=None):
        trans = Transaction.find(identification)
        if not trans.errors:
            new_trans = trans.void()
            transaction_was_successful.send(sender=self, 
                                            type="void",
                                            response=trans)
            return{'status': "SUCCESS", "response": new_trans}
        transaction_was_unsuccessful.send(sender=self, 
                                          type="void",
                                          response=trans)
        return{"status": "FAILURE", "response": trans}

    def credit(self, money, identification, options=None):
        trans = Transaction.find(identification)
        if not trans.errors:
            new_trans = trans.reverse(money)
            transaction_was_successful.send(sender=self, 
                                            type="credit",
                                            response=trans)
            return{'status': "SUCCESS", "response": new_trans}
        transaction_was_unsuccessful.send(sender=self, 
                                          type="credit",
                                          response=trans)
        return{"status": "FAILURE", "response": trans}

    def store(self, credit_card, options=None):
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            payment_method = PaymentMethod.create(credit_card.number, 
                                                        credit_card.verification_value, 
                                                        credit_card.month, credit_card.year)
        else:
            # Using the token which has to be retained
            payment_method = PaymentMethod.find(credit_card)
            if payment_method.errors:
                transaction_was_unsuccessful.send(sender=self, 
                                                  type="store",
                                                  response=payment_method)
                return {'status': 'FAILURE', 'response': payment_method}
        response = payment_method.retain()
        if response.errors:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="store",
                                              response=response)
            return {'status': 'FAILURE', 'response': response}
        transaction_was_successful.send(sender=self, 
                                        type="store",
                                        response=response)
        return {'status': 'SUCCESS', 'response': response}

    def unstore(self, identification, options=None):
        payment_method = PaymentMethod.find(identification)
        if payment_method.errors:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="unstore",
                                              response=payment_method)
            return {"status": "FAILURE", "response": payment_method}
        payment_method = payment_method.redact()
        if payment_method.errors:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="unstore",
                                              response=payment_method)
            return {"status": "FAILURE", "response": payment_method}
        transaction_was_successful.send(sender=self, 
                                        type="unstore",
                                        response=payment_method)
        return {"status": "SUCCESS", "response": payment_method}



from django.test import TestCase
from billing import get_gateway, CreditCard
from billing.signals import *
from billing.models import AuthorizeAIMResponse
from billing.gateway import CardNotSupported
from billing.utils.credit_card import Visa

class AuthorizeNetAIMGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("authorize_net")
        self.merchant.test_mode = True
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2011, 
                                      number="4222222222222", 
                                      verification_value="100")

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported, 
                          lambda : self.merchant.purchase(1000, self.credit_card))

    def testCardValidated(self):
        self.merchant.test_mode = False
        self.credit_card.number = "4222222222222123"
        self.assertFalse(self.merchant.validate_card(self.credit_card))

    def testCardType(self):
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        # In test mode, the transaction ID from Authorize.net is 0
        self.assertEquals(resp["response"].transaction_id, "0")
        self.assertTrue(isinstance(resp["response"], AuthorizeAIMResponse)) 

    def testPaymentSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_successful.connect(receive)

        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(received_signals, [transaction_was_successful])

    def testPaymentUnSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_unsuccessful.connect(receive)

        resp = self.merchant.purchase(6, self.credit_card)
        self.assertEquals(received_signals, [transaction_was_unsuccessful])

    def testCreditCardExpired(self):
        resp = self.merchant.purchase(8, self.credit_card)
        self.assertNotEquals(resp["status"], "SUCCESS")



from django.test import TestCase
from django.utils.html import strip_spaces_between_tags
from billing import get_integration
from django.template import Template, Context
from django.conf import settings

class GoogleCheckoutTestCase(TestCase):
    def setUp(self):
        self.gc = get_integration("google_checkout")
        fields = {"items": [{
                    "name": "name of the item",
                    "description": "Item description",
                    "amount": 1,
                    "id": "999AXZ", 
                    "currency": "USD",
                    "quantity": 1,
                    }],
                  "return_url": "http://127.0.0.1:8000/offsite/google-checkout/",
                  }
        self.gc.add_fields(fields)

    def testFormGen(self):
        tmpl = Template("{% load google_checkout from google_checkout_tags %}{% google_checkout obj %}")
        form = tmpl.render(Context({"obj": self.gc}))
        pregen_form = """<form action="https://sandbox.google.com/checkout/api/checkout/v2/checkout/Merchant/%(mid)s" method="post"><input type="hidden" name="cart" value="PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz48Y2hlY2tvdXQtc2hvcHBpbmctY2FydCB4bWxucz0iaHR0cDovL2NoZWNrb3V0Lmdvb2dsZS5jb20vc2NoZW1hLzIiPjxzaG9wcGluZy1jYXJ0PjxpdGVtcz48aXRlbT48aXRlbS1uYW1lPm5hbWUgb2YgdGhlIGl0ZW08L2l0ZW0tbmFtZT48aXRlbS1kZXNjcmlwdGlvbj5JdGVtIGRlc2NyaXB0aW9uPC9pdGVtLWRlc2NyaXB0aW9uPjx1bml0LXByaWNlIGN1cnJlbmN5PSJVU0QiPjE8L3VuaXQtcHJpY2U+PHF1YW50aXR5PjE8L3F1YW50aXR5PjxtZXJjaGFudC1pdGVtLWlkPjk5OUFYWjwvbWVyY2hhbnQtaXRlbS1pZD48L2l0ZW0+PC9pdGVtcz48L3Nob3BwaW5nLWNhcnQ+PC9jaGVja291dC1zaG9wcGluZy1jYXJ0Pg==" /><input type="hidden" name="signature" value="3jkvhENlILC3GTVNrXwmvldds4U=" /><input type="image" name="Google Checkout" alt="Fast checkout through Google" src="http://sandbox.google.com/checkout/buttons/checkout.gif?merchant_id=%(mid)s&amp;w=180&amp;h=46&amp;style=white&amp;variant=text&amp;loc=en_US" height="46" width="180" /></form>""" %({"mid": settings.MERCHANT_SETTINGS['google_checkout']['MERCHANT_ID']})
        self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())



from django.test import TestCase
from billing.utils.credit_card import CreditCard
from billing import get_gateway, GatewayNotConfigured, \
    get_integration, IntegrationNotConfigured
from django.conf import settings
from django.template import Template, Context, TemplateSyntaxError

class MerchantTestCase(TestCase):
    def testCorrectClassLoading(self):
        gateway = get_gateway("authorize_net")
        self.assertEquals(gateway.display_name, "Authorize.Net")

    def testSettingAttributes(self):
        self.assertTrue(getattr(settings, "MERCHANT_SETTINGS", None) != None)
        self.assertTrue(isinstance(settings.MERCHANT_SETTINGS, dict))

    def testRaiseExceptionNotConfigured(self):
        original_settings = settings.MERCHANT_SETTINGS
        settings.MERCHANT_SETTINGS = {
            "google_checkout": {
                "MERCHANT_ID" : '' ,
                "MERCHANT_KEY" : ''
                }
            }

        # Test if we can import any other gateway or integration
        self.assertRaises(IntegrationNotConfigured, lambda: get_integration("stripe"))
        self.assertRaises(GatewayNotConfigured, lambda: get_gateway("authorize_net"))
        settings.MERCHANT_SETTINGS = original_settings

    def testTemplateTagLoad(self):
        original_settings = settings.MERCHANT_SETTINGS
        settings.MERCHANT_SETTINGS = {
            "google_checkout": {
                "MERCHANT_ID" : '' ,
                "MERCHANT_KEY" : ''
                }
            }

        # Raises TemplateSyntaxError: Invalid Block Tag
        self.assertRaises(TemplateSyntaxError, lambda: Template("{% load google_checkout from google_checkout_tags %}{% stripe obj %}"))

        tmpl = Template("{% load google_checkout from google_checkout_tags %}{% google_checkout obj %}")
        gc = get_integration("google_checkout")
        fields = {"items": [{
                    "name": "name of the item",
                    "description": "Item description",
                    "amount": 1,
                    "id": "999AXZ", 
                    "currency": "USD",
                    "quantity": 1,
                    }],
                  "return_url": "http://127.0.0.1:8000/offsite/google-checkout/",
                  }
        gc.add_fields(fields)
        self.assertTrue(len(tmpl.render(Context({"obj": gc}))) > 0)

        settings.MERCHANT_SETTINGS = original_settings



import datetime

from paypal.pro.helpers import PayPalWPP
from paypal.pro.exceptions import PayPalFailure

from django.conf import settings

from billing import Gateway
from billing.utils.credit_card import Visa, MasterCard, AmericanExpress, Discover
from billing.signals import *

class PayPalGateway(Gateway):
    default_currency = "USD"
    supported_countries = ["US"]
    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    homepage_url = "https://merchant.paypal.com/us/cgi-bin/?&cmd=_render-content&content_ID=merchant/wp_pro"
    display_name = "PayPal Website Payments Pro"

    def __init__(self):
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("pay_pal"):
            raise GatewayNotConfigured("The '%s' gateway is not correctly "
                                       "configured." % self.display_name)
        pay_pal_settings = merchant_settings["pay_pal"]

    @property
    def service_url(self):
        # Implemented in django-paypal
        raise NotImplementedError
    
    def purchase(self, money, credit_card, options=None):
        """Using PAYPAL DoDirectPayment, charge the given
        credit card for specified money"""
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")

        params = {}
        params['creditcardtype'] = credit_card.card_type.card_name
        params['acct'] = credit_card.number
        params['expdate'] = '%02d%04d' % (credit_card.month, credit_card.year)
        params['cvv2'] = credit_card.verification_value
        params['ipaddress'] = options['request'].META.get("REMOTE_ADDR", "")
        params['amt'] = money

        if options.get("email"):
            params['email'] = options["email"]
        
        address = options.get("billing_address", {})
        first_name = None
        last_name = None
        try:
            first_name, last_name = address.get("name", "").split(" ")
        except ValueError:
            pass
        params['firstname'] = first_name or credit_card.first_name
        params['lastname'] = last_name or credit_card.last_name
        params['street'] = address.get("address1", '')
        params['street2'] = address.get("address2", "")
        params['city'] = address.get("city", '')
        params['state'] = address.get("state", '')
        params['countrycode'] = address.get("country", '')
        params['zip'] = address.get("zip", '')
        params['phone'] = address.get("phone", "")

        shipping_address = options.get("shipping_address", None)
        if shipping_address:
            params['shiptoname'] = shipping_address["name"]
            params['shiptostreet'] = shipping_address["address1"]
            params['shiptostreet2'] = shipping_address.get("address2", "")
            params['shiptocity'] = shipping_address["city"]
            params['shiptostate'] = shipping_address["state"]
            params['shiptocountry'] = shipping_address["country"]
            params['shiptozip'] = shipping_address["zip"]
            params['shiptophonenum'] = shipping_address.get("phone", "")
        
        wpp = PayPalWPP(options['request']) 
        try:
            response = wpp.doDirectPayment(params)
            transaction_was_successful.send(sender=self,
                                            type="purchase",
                                            response=response)
        except PayPalFailure, e:
            transaction_was_unsuccessful.send(sender=self,
                                              type="purchase",
                                              response=e)
            # Slight skewness because of the way django-paypal
            # is implemented.
            return {"status": "FAILURE", "response": e}
        return {"status": response.ack.upper(), "response": response}

    def authorize(self, money, credit_card, options = None):
        if not options:
            options = {}
        if not self.validate_card(credit_card):
            raise InvalidCard("Invalid Card")
        raise NotImplementedError

    def capture(self, money, authorization, options = None):
        raise NotImplementedError

    def void(self, identification, options = None):
        raise NotImplementedError

    def credit(self, money, identification, options = None):
        raise NotImplementedError

    def recurring(self, money, creditcard, options = None):
        if not options:
            options = {}
        params = {}
        params['profilestartdate'] = options.get('startdate') or datetime.datetime.now().strftime("%Y-%m-%dT00:00:00Z")
        params['startdate'] = options.get('startdate') or datetime.datetime.now().strftime("%m%Y")
        params['billingperiod'] = options.get('billingperiod') or 'Month'
        params['billingfrequency'] = options.get('billingfrequency') or '1'
        params['amt'] = money
        params['desc'] = 'description of the billing'
        
        params['creditcardtype'] = creditcard.card_type.card_name
        params['acct'] = creditcard.number
        params['expdate'] = '%02d%04d' % (creditcard.month, creditcard.year)
        params['firstname'] = creditcard.first_name
        params['lastname'] = creditcard.last_name

        wpp = PayPalWPP(options.get('request', {}))
        try:
            response = wpp.createRecurringPaymentsProfile(params, direct=True)
            transaction_was_successful.send(sender=self,
                                            type="purchase",
                                            response=response)
        except PayPalFailure, e:
            transaction_was_unsuccessful.send(sender=self,
                                              type="purchase",
                                              response=e)
            # Slight skewness because of the way django-paypal
            # is implemented.
            return {"status": "FAILURE", "response": e}
        return {"status": response.ack.upper(), "response": response}
       
        
    def store(self, creditcard, options = None):
        raise NotImplementedError

    def unstore(self, identification, options = None):
        raise NotImplementedError



# List taken from http://www.iso.org/iso/english_country_names_and_code_elements

COUNTRY_CODE = {
    "AFGHANISTAN" : "AF",
    "ÅLAND ISLANDS" : "AX",
    "ALBANIA" : "AL",
    "ALGERIA" : "DZ",
    "AMERICAN SAMOA" : "AS",
    "ANDORRA" : "AD",
    "ANGOLA" : "AO",
    "ANGUILLA" : "AI",
    "ANTARCTICA" : "AQ",
    "ANTIGUA AND BARBUDA" : "AG",
    "ARGENTINA" : "AR",
    "ARMENIA" : "AM",
    "ARUBA" : "AW",
    "AUSTRALIA" : "AU",
    "AUSTRIA" : "AT",
    "AZERBAIJAN" : "AZ",
    "BAHAMAS" :	"BS",
    "BAHRAIN" :	"BH",
    "BANGLADESH" : "BD",
    "BARBADOS" : "BB",
    "BELARUS" :	"BY",
    "BELGIUM" :	"BE",
    "BELIZE" :	"BZ",
    "BENIN" : "BJ",
    "BERMUDA" :	"BM",
    "BHUTAN" : "BT",
    "BOLIVIA, PLURINATIONAL STATE OF" : "BO",
    "BOSNIA AND HERZEGOVINA" : "BA",
    "BOTSWANA" : "BW",
    "BOUVET ISLAND" : "BV",
    "BRAZIL" : "BR",
    "BRITISH INDIAN OCEAN TERRITORY" : "IO",
    "BRUNEI DARUSSALAM" : "BN",
    "BULGARIA" : "BG",
    "BURKINA FASO" : "BF",
    "BURUNDI" : "BI",
    "CAMBODIA" : "KH",
    "CAMEROON" : "CM",
    "CANADA" : "CA",
    "CAPE VERDE" : "CV",
    "CAYMAN ISLANDS" : "KY",
    "CENTRAL AFRICAN REPUBLIC" : "CF",
    "CHAD" : "TD",
    "CHILE" : "CL",
    "CHINA" : "CN",
    "CHRISTMAS ISLAND" : "CX",
    "COCOS (KEELING) ISLANDS" : "CC",
    "COLOMBIA" : "CO",
    "COMOROS" : "KM",
    "CONGO" : "CG",
    "CONGO, THE DEMOCRATIC REPUBLIC OF THE" : "CD",
    "COOK ISLANDS" : "CK",
    "COSTA RICA" : "CR",
    "CÔTE D'IVOIRE" : "CI",
    "CROATIA" : "HR",
    "CUBA" : "CU",
    "CYPRUS" : "CY",
    "CZECH REPUBLIC" : "CZ",
    "DENMARK" : "DK",
    "DJIBOUTI" : "DJ",
    "DOMINICA" : "DM",
    "DOMINICAN REPUBLIC" : "DO",
    "ECUADOR" : "EC",
    "EGYPT" : "EG",
    "EL SALVADOR" : "SV",
    "EQUATORIAL GUINEA" : "GQ",
    "ERITREA" : "ER",
    "ESTONIA" : "EE",
    "ETHIOPIA" : "ET",
    "FALKLAND ISLANDS (MALVINAS)" : "FK",
    "FAROE ISLANDS" : "FO",
    "FIJI" : "FJ",
    "FINLAND" : "FI",
    "FRANCE" : "FR",
    "FRENCH GUIANA" : "GF",
    "FRENCH POLYNESIA" : "PF",
    "FRENCH SOUTHERN TERRITORIES" : "TF",
    "GABON" : "GA",
    "GAMBIA" : "GM",
    "GEORGIA" : "GE",
    "GERMANY" : "DE",
    "GHANA" : "GH",
    "GIBRALTAR" : "GI",
    "GREECE" : "GR",
    "GREENLAND" : "GL",
    "GRENADA" : "GD",
    "GUADELOUPE" : "GP",
    "GUAM" : "GU",
    "GUATEMALA" : "GT",
    "GUERNSEY" : "GG",
    "GUINEA" : "GN",
    "GUINEA-BISSAU" : "GW",
    "GUYANA" : "GY",
    "HAITI" : "HT",
    "HEARD ISLAND AND MCDONALD ISLANDS" : "HM",
    "HOLY SEE (VATICAN CITY STATE)" : "VA",
    "HONDURAS" : "HN",
    "HONG KONG" : "HK",
    "HUNGARY" : "HU",
    "ICELAND" : "IS",
    "INDIA" : "IN",
    "INDONESIA" : "ID",
    "IRAN, ISLAMIC REPUBLIC OF" : "IR",
    "IRAQ" : "IQ",
    "IRELAND" : "IE",
    "ISLE OF MAN" : "IM",
    "ISRAEL" : "IL",
    "ITALY" : "IT",
    "JAMAICA" : "JM",
    "JAPAN" : "JP",
    "JERSEY" : "JE",
    "JORDAN" : "JO",
    "KAZAKHSTAN" : "KZ",
    "KENYA" : "KE",
    "KIRIBATI" : "KI",
    "KOREA, DEMOCRATIC PEOPLE'S REPUBLIC OF" : "KP",
    "KOREA, REPUBLIC OF" : "KR",
    "KUWAIT" : "KW",
    "KYRGYZSTAN" : "KG",
    "LAO PEOPLE'S DEMOCRATIC REPUBLIC" : "LA",
    "LATVIA" : "LV",
    "LEBANON" : "LB",
    "LESOTHO" : "LS",
    "LIBERIA" : "LR",
    "LIBYAN ARAB JAMAHIRIYA" : "LY",
    "LIECHTENSTEIN" : "LI",
    "LITHUANIA" : "LT",
    "LUXEMBOURG" : "LU",
    "MACAO" : "MO",
    "MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF" : "MK",
    "MADAGASCAR" : "MG",
    "MALAWI" : "MW",
    "MALAYSIA" : "MY",
    "MALDIVES" : "MV",
    "MALI" : "ML",
    "MALTA" : "MT",
    "MARSHALL ISLANDS" : "MH",
    "MARTINIQUE" : "MQ",
    "MAURITANIA" : "MR",
    "MAURITIUS" : "MU",
    "MAYOTTE" : "YT",
    "MEXICO" : "MX",
    "MICRONESIA, FEDERATED STATES OF" : "FM",
    "MOLDOVA, REPUBLIC OF" : "MD",
    "MONACO" : "MC",
    "MONGOLIA" : "MN",
    "MONTENEGRO" : "ME",
    "MONTSERRAT" : "MS",
    "MOROCCO" : "MA",
    "MOZAMBIQUE" : "MZ",
    "MYANMAR" : "MM",
    "NAMIBIA" : "NA",
    "NAURU" : "NR",
    "NEPAL" : "NP",
    "NETHERLANDS" : "NL",
    "NETHERLANDS ANTILLES" : "AN",
    "NEW CALEDONIA" : "NC",
    "NEW ZEALAND" : "NZ",
    "NICARAGUA" : "NI",
    "NIGER" : "NE",
    "NIGERIA" : "NG",
    "NIUE" : "NU",
    "NORFOLK ISLAND" : "NF",
    "NORTHERN MARIANA ISLANDS" : "MP",
    "NORWAY" : "NO",
    "OMAN" : "OM",
    "PAKISTAN" : "PK",
    "PALAU" : "PW",
    "PALESTINIAN TERRITORY, OCCUPIED" : "PS",
    "PANAMA" : "PA",
    "PAPUA NEW GUINEA" : "PG",
    "PARAGUAY" : "PY",
    "PERU" : "PE",
    "PHILIPPINES" : "PH",
    "PITCAIRN" : "PN",
    "POLAND" : "PL",
    "PORTUGAL" : "PT",
    "PUERTO RICO" : "PR",
    "QATAR" : "QA",
    "RÉUNION" : "RE",
    "ROMANIA" : "RO",
    "RUSSIAN FEDERATION" : "RU",
    "RWANDA" : "RW",
    "SAINT BARTHÉLEMY" : "BL",
    "SAINT HELENA, ASCENSION AND TRISTAN DA CUNHA" : "SH",
    "SAINT KITTS AND NEVIS" : "KN",
    "SAINT LUCIA" : "LC",
    "SAINT MARTIN" : "MF",
    "SAINT PIERRE AND MIQUELON" : "PM",
    "SAINT VINCENT AND THE GRENADINES" : "VC",
    "SAMOA" : "WS",
    "SAN MARINO" : "SM",
    "SAO TOME AND PRINCIPE" : "ST",
    "SAUDI ARABIA" : "SA",
    "SENEGAL" : "SN",
    "SERBIA" : "RS",
    "SEYCHELLES" : "SC",
    "SIERRA LEONE" : "SL",
    "SINGAPORE" : "SG",
    "SLOVAKIA" : "SK",
    "SLOVENIA" : "SI",
    "SOLOMON ISLANDS" : "SB",
    "SOMALIA" : "SO",
    "SOUTH AFRICA" : "ZA",
    "SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS" : "GS",
    "SPAIN" : "ES",
    "SRI LANKA" : "LK",
    "SUDAN" : "SD",
    "SURINAME" : "SR",
    "SVALBARD AND JAN MAYEN" : "SJ",
    "SWAZILAND" : "SZ",
    "SWEDEN" : "SE",
    "SWITZERLAND" : "CH",
    "SYRIAN ARAB REPUBLIC" : "SY",
    "TAIWAN, PROVINCE OF CHINA" : "TW",
    "TAJIKISTAN" : "TJ",
    "TANZANIA, UNITED REPUBLIC OF" : "TZ",
    "THAILAND" : "TH",
    "TIMOR-LESTE" : "TL",
    "TOGO" : "TG",
    "TOKELAU" : "TK",
    "TONGA" : "TO",
    "TRINIDAD AND TOBAGO" : "TT",
    "TUNISIA" : "TN",
    "TURKEY" : "TR",
    "TURKMENISTAN" : "TM",
    "TURKS AND CAICOS ISLANDS" : "TC",
    "TUVALU" : "TV",
    "UGANDA" : "UG",
    "UKRAINE" : "UA",
    "UNITED ARAB EMIRATES" : "AE",
    "UNITED KINGDOM" : "GB",
    "UNITED STATES" : "US",
    "UNITED STATES MINOR OUTLYING ISLANDS" : "UM",
    "URUGUAY" : "UY",
    "UZBEKISTAN" : "UZ",
    "VANUATU" : "VU",
    "VENEZUELA, BOLIVARIAN REPUBLIC OF" : "VE",
    "VIET NAM" : "VN",
    "VIRGIN ISLANDS, BRITISH" :	"VG",
    "VIRGIN ISLANDS, U.S" : "VI",
    "WALLIS AND FUTUNA" : "WF",
    "WESTERN SAHARA" : "EH",
    "YEMEN" : "YE",
    "ZAMBIA" : "ZM",
    "ZIMBABWE" : "ZW",
}



from billing.integration import Integration, IntegrationNotConfigured
from django.conf import settings
from boto.fps.connection import FPSConnection
from django.conf.urls.defaults import patterns, url
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.http import (HttpResponseForbidden, 
                         HttpResponseRedirect, 
                         HttpResponse)
from billing.signals import (transaction_was_successful,
                             transaction_was_unsuccessful)
from django.core.urlresolvers import reverse
from billing.models import AmazonFPSResponse
import urlparse, urllib, time, datetime

FPS_PROD_API_ENDPOINT = "fps.amazonaws.com"
FPS_SANDBOX_API_ENDPOINT = "fps.sandbox.amazonaws.com"

csrf_exempt_m = method_decorator(csrf_exempt)
require_POST_m = method_decorator(require_POST)

class AmazonFpsIntegration(Integration):
    """
    Fields required:
    transactionAmount: Amount to be charged/authorized
    paymentReason: Description of the transaction
    paymentPage: Page to direct the user on completion/failure of transaction
    """

    display_name = "Amazon Flexible Payment Service"

    def __init__(self, options=None):
        if not options:
            options = {}
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("amazon_fps"):
            raise IntegrationNotConfigured("The '%s' integration is not correctly "
                                       "configured." % self.display_name)
        amazon_fps_settings = merchant_settings["amazon_fps"]
        self.aws_access_key = options.get("aws_access_key", None) or amazon_fps_settings['AWS_ACCESS_KEY']
        self.aws_secret_access_key = options.get("aws_secret_access_key", None) or amazon_fps_settings['AWS_SECRET_ACCESS_KEY']
        super(AmazonFpsIntegration, self).__init__(options=options)
        self.fps_connection = FPSConnection(self.aws_access_key, self.aws_secret_access_key, **options)

    @property
    def service_url(self):
        if self.test_mode:
            return FPS_SANDBOX_API_ENDPOINT
        return FPS_PROD_API_ENDPOINT

    @property
    def link_url(self):
        tmp_fields = self.fields.copy()
        tmp_fields.pop("aws_access_key", None)
        tmp_fields.pop("aws_secret_access_key", None)
        tmp_fields.pop("paymentPage", None)
        return self.fps_connection.make_url(tmp_fields.pop("returnURL"), 
                                            tmp_fields.pop("paymentReason"),
                                            tmp_fields.pop("pipelineName"),
                                            str(tmp_fields.pop("transactionAmount")),
                                            **tmp_fields)

    def purchase(self, amount, options=None):
        if not options:
            options = {}
        tmp_options = options.copy()
        permissible_options = ["senderTokenId", "recipientTokenId", 
            "chargeFeeTo", "callerReference", "senderReference", "recipientReference",
            "senderDescription", "recipientDescription", "callerDescription",
            "metadata", "transactionDate", "reserve"]
        tmp_options["senderTokenId"] = options["tokenID"]
        for key in options:
            if key not in permissible_options:
                tmp_options.pop(key)
        resp = self.fps_connection.pay(amount, tmp_options.pop("senderTokenId"), 
                                       callerReference=tmp_options.pop("callerReference"),
                                       **tmp_options)
        return {"status": resp[0].TransactionStatus, "response": resp[0]}

    def authorize(self, amount, options=None):
        if not options:
            options = {}
        options["reserve"] = True
        return self.purchase(amount, options)

    def capture(self, amount, options=None):
        if not options:
            options = {}
        assert "ReserveTransactionId" in options, "Expecting 'ReserveTransactionId' in options"
        resp = self.fps_connection.settle(options["ReserveTransactionId"], amount)
        return {"status": resp[0].TransactionStatus, "response": resp[0]}

    def credit(self, amount, options=None):
        if not options:
            options = {}
        assert "CallerReference" in options, "Expecting 'CallerReference' in options"
        assert "TransactionId" in options, "Expecting 'TransactionId' in options"
        resp = self.fps_connection.refund(options["CallerReference"],
                                          options["TransactionId"], 
                                          refundAmount=amount,
                                          callerDescription=options.get("description", None))
        return {"status": resp[0].TransactionStatus, "response": resp[0]}

    def void(self, identification, options=None):
        if not options:
            options = {}
        # Requires the TransactionID to be passed as 'identification'
        resp = self.fps_connection.cancel(identification, 
                                          options.get("description", None))
        return {"status": resp[0].TransactionStatus, "response": resp[0]}

    def get_urls(self):
        urlpatterns = patterns('',
           url(r'^fps-notify-handler/$', self.fps_ipn_handler, name="fps_ipn_handler"),
           url(r'^fps-return-url/$', self.fps_return_url, name="fps_return_url"),
                               )
        return urlpatterns

    @csrf_exempt_m
    @require_POST_m
    def fps_ipn_handler(self, request):
        uri = request.build_absolute_uri()
        parsed_url = urlparse.urlparse(uri)
        resp = self.fps_connection.verify_signature("%s://%s%s" %(parsed_url.scheme, 
                                                                  parsed_url.netloc, 
                                                                  parsed_url.path),
                                                    request.raw_post_data)
        if not resp[0].VerificationStatus == "Success":
            return HttpResponseForbidden()

        data = dict(map(lambda x: x.split("="), request.raw_post_data.split("&")))
        for (key, val) in data.iteritems():
            data[key] = urllib.unquote_plus(val)
        if AmazonFPSResponse.objects.filter(transactionId=data["transactionId"]).count():
            resp = AmazonFPSResponse.objects.get(transactionId=data["transactionId"])
        else:
            resp = AmazonFPSResponse()
        for (key, val) in data.iteritems():
            attr_exists = hasattr(resp, key)
            if attr_exists and not callable(getattr(resp, key, None)):
                if key == "transactionDate":
                    val = datetime.datetime(*time.localtime(float(val))[:6])
                setattr(resp, key, val)
        resp.save()
        if resp.statusCode == "Success":
            transaction_was_successful.send(sender=self.__class__, 
                                            type=data["operation"], 
                                            response=resp)
        else:
            if not "Pending" in resp.statusCode:
                transaction_was_unsuccessful.send(sender=self.__class__, 
                                                  type=data["operation"], 
                                                  response=resp)
        # Return a HttpResponse to prevent django from complaining
        return HttpResponse(resp.statusCode)

    def fps_return_url(self, request):
        uri = request.build_absolute_uri()
        parsed_url = urlparse.urlparse(uri)
        resp = self.fps_connection.verify_signature("%s://%s%s" %(parsed_url.scheme, 
                                                                  parsed_url.netloc, 
                                                                  parsed_url.path),
                                                    parsed_url.query)
        if not resp[0].VerificationStatus == "Success":
            return HttpResponseForbidden()

        return self.transaction(request)

    def transaction(self, request):
        """Has to be overridden by the subclasses"""
        raise NotImplementedError



from django.test import TestCase
from billing import get_gateway, CreditCard
from billing.gateway import CardNotSupported
from billing.utils.credit_card import Visa
import stripe
from django.conf import settings


class StripeGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("stripe")
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2012,
                                      number="4242424242424242",
                                      verification_value="100")
        stripe.api_key = self.merchant.stripe.api_key
        self.stripe = stripe

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported,
                     lambda: self.merchant.purchase(1000, self.credit_card))

    def testCardType(self):
        self.credit_card.number = '4242424242424242'
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")

    def testStoreMissingCustomer(self):
        self.assertRaises(TypeError, self.merchant.store)

    def testStoreWithoutBillingAddress(self):
        resp = self.merchant.store(self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        self.assertEquals(resp["response"].active_card.exp_month, self.credit_card.month)
        self.assertEquals(resp["response"].active_card.exp_year, self.credit_card.year)
        self.assertTrue(getattr(resp["response"], "id"))
        self.assertTrue(getattr(resp["response"], "created"))

    def testUnstore(self):
        resp = self.merchant.store(self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.unstore(resp["response"].id)
        self.assertEquals(response["status"], "SUCCESS")

    def testRecurring1(self):
        plan_id = "test_plan"
        try:
            plan = self.stripe.Plan.retrieve(plan_id)
        except self.stripe.InvalidRequestError:
            response = self.stripe.Plan.create(
                amount=1000,
                interval='month',
                name="Test Plan",
                currency="usd",
                id=plan_id)
        options = {"plan_id": plan_id}
        resp = self.merchant.recurring(self.credit_card, options=options)
        self.assertEquals(resp["status"], "SUCCESS")
        subscription = resp["response"].subscription
        self.assertEquals(subscription.status, "active")

    def testCredit(self):
        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        identification = resp["response"].id
        resp = self.merchant.credit(identification=identification)
        self.assertEquals(resp["status"], "SUCCESS")

    def testAuthorizeAndCapture(self):
        resp = self.merchant.authorize(100, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.capture(50, resp["response"].id)
        self.assertEquals(response["status"], "SUCCESS")

    def testPurchaseWithToken(self):
        # Somewhat similar to capture but testing for the 
        # purpose of the stripe integration
        resp = self.merchant.authorize(1, self.credit_card)
        resp = self.merchant.purchase(1, resp["response"].id)
        self.assertEquals(resp["status"], "SUCCESS")





from xml.dom.minidom import parse, parseString
import os

class NotTextNodeError:
    pass

def getTextFromNode(node):
    """
    scans through all children of node and gathers the
    text. if node has non-text child-nodes, then
    NotTextNodeError is raised.
    """
    t = ""
    for n in node.childNodes:
        if n.nodeType == n.TEXT_NODE:
            t += n.nodeValue
        else:
            raise NotTextNodeError
    return t

def nodeToDic(node):
    """
    nodeToDic() scans through the children of node and makes a
    dictionary from the content.
    three cases are differentiated:
    - if the node contains no other nodes, it is a text-node
    and {nodeName:text} is merged into the dictionary.
    - if the node has the attribute "method" set to "true",
    then it's children will be appended to a list and this
    list is merged to the dictionary in the form: {nodeName:list}.
    - else, nodeToDic() will call itself recursively on
    the nodes children (merging {nodeName:nodeToDic()} to
    the dictionary).
    """
    dic = {}
    multlist = {} # holds temporary lists where there are multiple children
    multiple = False
    for n in node.childNodes:
        if n.nodeType != n.ELEMENT_NODE:
            continue
        # find out if there are multiple records
        if len(node.getElementsByTagName(n.nodeName)) > 1:
            multiple = True
            # and set up the list to hold the values
            if not multlist.has_key(n.nodeName):
                multlist[n.nodeName] = []
        else:
            multiple = False
        try:
            # text node
            text = getTextFromNode(n)
        except NotTextNodeError:
            if multiple:
                # append to our list
                multlist[n.nodeName].append(nodeToDic(n))
                dic.update({n.nodeName:multlist[n.nodeName]})
                continue
            else:
                # 'normal' node
                dic.update({n.nodeName:nodeToDic(n)})
                continue
        # text node
        if multiple:
            multlist[n.nodeName].append(text)
            dic.update({n.nodeName:multlist[n.nodeName]})
        else:
            dic.update({n.nodeName:text})
    return dic 

def readConfig(filename):
    dom = parseString(open(filename).read())
    return nodeToDic(dom)

if __name__ == "__main__":
    import sys
    import pprint
    pprint.pprint(readConfig(sys.argv[1]))




from django.contrib import admin

import billing.models as billing_models

admin.site.register(billing_models.GCNewOrderNotification)
admin.site.register(billing_models.AuthorizeAIMResponse)
admin.site.register(billing_models.WorldPayResponse)
admin.site.register(billing_models.AmazonFPSResponse)




from django.db import models

class GCNewOrderNotification(models.Model):
    notify_type         = models.CharField(max_length=255, blank=True)
    serial_number       = models.CharField(max_length=255)
    google_order_number = models.CharField(max_length=255)
    buyer_id            = models.CharField(max_length=255)
    
    # Buyer Shipping Address details
    shipping_contact_name = models.CharField(max_length=255, blank=True)
    shipping_address1     = models.CharField(max_length=255, blank=True)
    shipping_address2     = models.CharField(max_length=255, blank=True)
    shipping_city         = models.CharField(max_length=255, blank=True)
    shipping_postal_code  = models.CharField(max_length=255, blank=True)
    shipping_region       = models.CharField(max_length=255, blank=True)
    shipping_country_code = models.CharField(max_length=255, blank=True)
    shipping_email        = models.EmailField()
    shipping_company_name = models.CharField(max_length=255, blank=True)
    shipping_fax          = models.CharField(max_length=255, blank=True)
    shipping_phone        = models.CharField(max_length=255, blank=True)
    
    # Buyer Billing Address details
    billing_contact_name = models.CharField(max_length=255, blank=True)
    billing_address1     = models.CharField(max_length=255, blank=True)
    billing_address2     = models.CharField(max_length=255, blank=True)
    billing_city         = models.CharField(max_length=255, blank=True)
    billing_postal_code  = models.CharField(max_length=255, blank=True)
    billing_region       = models.CharField(max_length=255, blank=True)
    billing_country_code = models.CharField(max_length=255, blank=True)
    billing_email        = models.EmailField()
    billing_company_name = models.CharField(max_length=255, blank=True)
    billing_fax          = models.CharField(max_length=255, blank=True)
    billing_phone        = models.CharField(max_length=255, blank=True)

    # Buyer marketing preferences, bool marketing email allowed
    marketing_email_allowed = models.BooleanField(default=False)
    
    num_cart_items = models.IntegerField()
    cart_items     = models.TextField()
    
    # Order Adjustment details
    total_tax                 = models.DecimalField(max_digits=16, decimal_places=2, blank=True, null=True)
    total_tax_currency        = models.CharField(max_length=255, blank=True)
    adjustment_total          = models.DecimalField(max_digits=16, decimal_places=2, blank=True, null=True)
    adjustment_total_currency = models.CharField(max_length=255, blank=True)
    
    order_total               = models.DecimalField(max_digits=16, decimal_places=2, blank=True, null=True)
    order_total_currency      = models.CharField(max_length=255, blank=True)
    
    financial_order_state     = models.CharField(max_length=255, blank=True)
    fulfillment_order_state   = models.CharField(max_length=255, blank=True)
    
    # u'timestamp': [u'2010-10-04T14:05:39.868Z'], 
    # timestamp = models.DateTimeField(blank=True, null=True)
    timestamp = models.CharField(max_length=64, blank=True, null=True)
    
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = __name__.split(".")[0]



from django import forms


class SamuraiForm(forms.Form):
    credit_card__number = forms.CharField()
    credit_card__cvc = forms.CharField(max_length=4)
    credit_card__expiration_month = forms.CharField(max_length=7)
    credit_card__expiration_year = forms.CharField(max_length=7)



"""
Braintree Payments Transparent Redirect Tests.
"""
from django.test import TestCase
from django.utils.html import strip_spaces_between_tags
from billing import get_integration
from django.template import Template, Context
from django.conf import settings

class BraintreePaymentsIntegrationTestCase(TestCase):
    urls = "billing.tests.test_urls"

    def setUp(self):
        self.bp = get_integration("braintree_payments")
        fields = {
            "transaction": {
                "type": "sale",
                "amount": "10.00",
                "order_id": 1,
                "customer": {
                    "first_name": "John",
                    "last_name": "Doe",
                    "email": "john.doe@example.com",
                    },
                }
            }
        self.bp.add_fields(fields)

    def testFormFields(self):
        self.assertEquals(self.bp.fields, {"transaction__type" : "sale",
                                           "transaction__amount": "10.00",
                                           "transaction__order_id": 1,
                                           "transaction__customer__first_name": "John",
                                           "transaction__customer__last_name": "Doe",
                                           "transaction__customer__email": "john.doe@example.com"})

    # Need to think about the tests below because they are dynamic because
    # of the hashes and the timestamps.
    # def testFormGen(self):
    #     tmpl = Template("{% load braintree_payments from braintree_payments_tags %}{% braintree_payments obj %}")
    #     form = tmpl.render(Context({"obj": self.bp}))
    #     print self.bp.generate_form()
    #     pregen_form = """""" %(settings.BRAINTREE_MERCHANT_ACCOUNT_ID)
    #     self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())

    # def testFormGen2(self):
    #     tmpl = Template("{% load braintree_payments from braintree_payments_tags %}{% braintree_payments obj %}")
    #     form = tmpl.render(Context({"obj": self.bp}))
    #     pregen_form = u"""%s""" %(settings.BRAINTREE_MERCHANT_ACCOUNT_ID)
    #     self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())




import re
import datetime

class InvalidCard(Exception):
    pass

class CardNotSupported(Exception):
    pass

class CreditCard(object):
    # The regexp attribute should be overriden by the subclasses.
    # Attribute value should be a regexp instance
    regexp = None
    # Has to be set by the user after calling `validate_card`
    # method on the gateway
    card_type = None
    # Required mainly for PayPal. PayPal expects to be sent
    # the card type also with the requests.
    card_name = None

    def __init__(self, **kwargs):
        self.first_name = kwargs["first_name"]
        self.last_name = kwargs["last_name"]
        self.month = int(kwargs["month"])
        self.year = int(kwargs["year"])
        self.number = kwargs["number"]
        self.verification_value = kwargs["verification_value"]
    
    def is_luhn_valid(self):
        """Checks the validity of card number by using Luhn Algorithm. 
        Please see http://en.wikipedia.org/wiki/Luhn_algorithm for details."""
        num = [int(x) for x in str(self.number)]
        return not sum(num[::-2] + [sum(divmod(d * 2, 10)) for d in num[-2::-2]]) % 10
    
    def is_expired(self):
        """Check whether the credit card is expired or not"""
        return datetime.date.today() > datetime.date(self.year, self.month, 1)
    
    def valid_essential_attributes(self):
        """Validate that all the required attributes of card are given"""
        return self.first_name and \
               self.last_name and \
               self.month and \
               self.year and \
               self.number and \
               self.verification_value and True
    
    def is_valid(self):
        """Check the validity of the card"""
        return self.is_luhn_valid() and \
               not self.is_expired() and \
               self.valid_essential_attributes()
    
    @property
    def expire_date(self):
        """Returns the expiry date of the card in MM-YYYY format"""
        return '%02d-%04d' % (self.month, self.year)
    
    @property
    def name(self):
        """Concat first name and last name of the card holder"""
        return '%s %s' % (self.first_name, self.last_name)


class Visa(CreditCard):
    card_name = "Visa"
    regexp = re.compile('^4\d{12}(\d{3})?$')

class MasterCard(CreditCard):
    card_name = "MasterCard"
    regexp = re.compile('^(5[1-5]\d{4}|677189)\d{10}$')

class Discover(CreditCard):
    card_name = "Discover"
    regexp = re.compile('^(6011|65\d{2})\d{12}$')

class AmericanExpress(CreditCard):
    card_name = "Amex"
    regexp = re.compile('^3[47]\d{13}$')

class DinersClub(CreditCard):
    card_name = "DinersClub"
    regexp = re.compile('^3(0[0-5]|[68]\d)\d{11}$')

class JCB(CreditCard):
    card_name = "JCB"
    regexp = re.compile('^35(28|29|[3-8]\d)\d{12}$')

class Switch(CreditCard):
    # Debit Card
    card_name = "Switch"
    regexp = re.compile('^6759\d{12}(\d{2,3})?$')

class Solo(CreditCard):
    # Debit Card
    card_name = "Solo"
    regexp = re.compile('^6767\d{12}(\d{2,3})?$')

class Dankort(CreditCard):
    # Debit cum Credit Card
    card_name = "Dankort"
    regexp = re.compile('^5019\d{12}$')

class Maestro(CreditCard):
    # Debit Card
    card_name = "Maestro"
    regexp = re.compile('^(5[06-8]|6\d)\d{10,17}$')

class Forbrugsforeningen(CreditCard):
    card_name = "Forbrugsforeningen"
    regexp = re.compile('^600722\d{10}$')

class Laser(CreditCard):
    # Debit Card
    card_name = "Laser"
    regexp = re.compile('^(6304|6706|6771|6709)\d{8}(\d{4}|\d{6,7})?$')

# A few helpful (probably) attributes
all_credit_cards = [Visa, MasterCard, Discover, AmericanExpress,
                    DinersClub, JCB]

all_debit_cards  = [Switch, Solo, Dankort, Maestro,
                    Forbrugsforeningen, Laser]

all_cards = all_credit_cards + all_debit_cards




from django.db import models

# Response Codes
# APPROVED, DECLINED, ERROR, FRAUD_REVIEW = 1, 2, 3, 4
class AuthorizeAIMResponse(models.Model):
    RESPONSE_CODES = [
        (1, 'Approved'),
        (2, 'Declined'),
        (3, 'Error'),
        (4, 'Held for Review'),
    ]
    ADDRESS_VERIFICATION_RESPONSE = [
        ('A', 'Address(Street) matches,ZIP does not'),
        ('B', 'Address information not provided for AVS check'),
        ('E', 'AVS error'),
        ('G', 'Non-U.S. Card Issuing Bank'),
        ('N', 'No match on Address(Street) or ZIP'),
        ('P', 'AVS not applicable for this transactions'),
        ('R', 'Retry-System unavailable or timed out'),
        ('S', 'Service not supported by issuer'),
        ('U', 'Address information is unavailable'),
        ('W', 'Nine digit Zip matches, Address(Street) does not'),
        ('X', 'Address(Street) and nine digit ZIP match'),
        ('Y', 'Address(Street) and five digit ZIP match'),
        ('Z', 'Five digit Zip matches, Address(Street) does not'),
    ]
    CARD_CODE_RESPONSES = [
        ('', ''),
        ('M', 'Match'),
        ('N', 'No Match'),
        ('P', 'Not Processed'),
        ('S', 'Should have been present'),
        ('U', 'Issuer unable to process request'),
    ]
    response_code = models.IntegerField(choices=RESPONSE_CODES)
    response_reason_code = models.IntegerField(blank=True)
    response_reason_text = models.TextField(blank=True)
    authorization_code = models.CharField(max_length=8)
    address_verification_response = models.CharField(max_length='8', choices=ADDRESS_VERIFICATION_RESPONSE)
    transaction_id = models.CharField(max_length=64)
    invoice_number = models.CharField(max_length=64, blank=True)
    description = models.CharField(max_length=255, blank=True)
    amount = models.DecimalField(max_digits=16, decimal_places=2)
    method = models.CharField(max_length=255, blank=True)
    transaction_type = models.CharField(max_length=255, blank=True)
    customer_id = models.CharField(max_length=64, blank=True)

    first_name = models.CharField(max_length=64, blank=True)
    last_name = models.CharField(max_length=64, blank=True)
    company = models.CharField(max_length=64, blank=True)
    address = models.CharField(max_length=64, blank=True)
    city = models.CharField(max_length=64, blank=True)
    state = models.CharField(max_length=64, blank=True)
    zip_code = models.CharField(max_length=64, blank=True)
    country = models.CharField(max_length=64, blank=True)
    phone = models.CharField(max_length=64, blank=True)
    fax = models.CharField(max_length=64, blank=True)
    email = models.EmailField()

    shipping_first_name = models.CharField(max_length=64, blank=True)
    shipping_last_name = models.CharField(max_length=64, blank=True)
    shipping_company = models.CharField(max_length=64, blank=True)
    shipping_address = models.CharField(max_length=64, blank=True)
    shipping_city = models.CharField(max_length=64, blank=True)
    shipping_state = models.CharField(max_length=64, blank=True)
    shipping_zip_code = models.CharField(max_length=64, blank=True)
    shipping_country = models.CharField(max_length=64, blank=True)
 
    card_code_response = models.CharField(max_length='8', choices=CARD_CODE_RESPONSES, help_text=u'Card Code Verification response')

    class Meta:
        app_label = __name__.split(".")[0]



from django.test import TestCase
from billing import get_gateway, CreditCard
from billing.gateway import CardNotSupported
from billing.utils.credit_card import Visa
from samurai.payment_method import PaymentMethod


class SamuraiGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("samurai")
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2012,
                                      number="4111111111111111",
                                      verification_value="100")

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported,
                     lambda: self.merchant.purchase(1000, self.credit_card))

    def testCardType(self):
        self.credit_card.number = '4242424242424242'
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")

    def testStoreMissingCustomer(self):
        self.assertRaises(TypeError, self.merchant.store)

    def testCredit(self):
        resp = self.merchant.purchase(1000, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        identification = resp["response"].reference_id
        resp = self.merchant.credit(money=100, identification=identification)
        self.assertEquals(resp["status"], "SUCCESS")

    def testAuthorizeAndCapture(self):
        resp = self.merchant.authorize(1000, self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.capture(50, resp["response"].reference_id)
        self.assertEquals(response["status"], "SUCCESS")

    def testStoreWithoutBillingAddress(self):
        resp = self.merchant.store(self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        self.assertEquals(resp["response"].expiry_month, self.credit_card.month)
        self.assertEquals(resp["response"].expiry_year, self.credit_card.year)
        self.assertTrue(getattr(resp["response"], "payment_method_token"))
        self.assertTrue(getattr(resp["response"], "created_at"))

    def testUnstore(self):
        resp = self.merchant.store(self.credit_card)
        self.assertEquals(resp["status"], "SUCCESS")
        response = self.merchant.unstore(resp["response"].payment_method_token)
        self.assertEquals(response["status"], "SUCCESS")

    def testPurchaseWithToken(self):
        resp = PaymentMethod.create('4111111111111111', '111', '07', '14')
        resp = self.merchant.purchase(500, resp.payment_method_token)
        self.assertEquals(resp["status"], "SUCCESS")



from billing.integration import Integration
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.conf.urls.defaults import patterns
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from django.http import HttpResponse
from billing.models.world_pay_models import WorldPayResponse
from django.utils.decorators import method_decorator

RBS_HOSTED_URL_TEST = "https://select-test.wp3.rbsworldpay.com/wcc/purchase"
RBS_HOSTED_URL_LIVE = "https://secure.wp3.rbsworldpay.com/wcc/purchase"

# http://www.rbsworldpay.com/support/bg/index.php?page=development&sub=integration&c=WW

csrf_exempt_m = method_decorator(csrf_exempt)
require_POST_m = method_decorator(require_POST)

class WorldPayIntegration(Integration):
    """
    Fields required:
    instId: Installation ID provided by WorldPay
    cartId: Merchant specified unique id to identify user
    amount: Amount to be charged
    currency: ISO 3-character currency
    """
    display_name = "RBS World Pay"

    def __init__(self, options=None):
        if not options:
            options = {}
        super(WorldPayIntegration, self).__init__(options=options)
        if self.test_mode:
            self.fields.update({"testMode": 100})

    def get_urls(self):
        urlpatterns = patterns('',
           (r'^rbs-notify-handler/$', self.notify_handler),
                               )
        return urlpatterns

    @property
    def service_url(self):
        if self.test_mode:
            return RBS_HOSTED_URL_TEST
        return RBS_HOSTED_URL_LIVE

    @csrf_exempt_m
    @require_POST_m
    def notify_handler(self, request):
        post_data = request.POST.copy()
        data = {}

        resp_fields = {
            'instId': 'installation_id',
            'compName': 'company_name',
            'cartId': 'cart_id',
            'desc': 'description',
            'amount': 'amount',
            'currency': 'currency',
            'authMode': 'auth_mode',
            'testMode': 'test_mode',
            'transId': 'transaction_id',
            'transStatus': 'transaction_status',
            'transTime': 'transaction_time',
            'authAmount': 'auth_amount',
            'authCurrency': 'auth_currency',
            'authAmountString': 'auth_amount_string',
            'rawAuthMessage': 'raw_auth_message',
            'rawAuthCode': 'raw_auth_code',
            'name': 'name',
            'address': 'address',
            'postcode': 'post_code',
            'country': 'country_code',
            'countryString': 'country',
            'tel': 'phone',
            'fax': 'fax',
            'email': 'email',
            'futurePayId': 'future_pay_id',
            'cardType': 'card_type',
            'ipAddress': 'ip_address',
            }

        for (key, val) in resp_fields.iteritems():
            data[val] = post_data.get(key, '')

        try:
            resp = WorldPayResponse.objects.create(**data)
            # TODO: Make the type more generic
            transaction_was_successful.send(sender=self.__class__, type="purchase", response=resp)
            status = "SUCCESS"
        except:
            transaction_was_unsuccessful.send(sender=self.__class__, type="purchase", response=post_data)
            status = "FAILURE"
        
        return HttpResponse(status)



from django.test import TestCase
from billing import get_gateway, get_integration, CreditCard
from billing.signals import *
from paypal.pro.models import PayPalNVP
from billing.gateway import CardNotSupported
from billing.utils.credit_card import Visa
from paypal.pro.tests import RequestFactory
from django.template import Template, Context
from django.utils.html import strip_spaces_between_tags
from django.core.urlresolvers import reverse
import datetime

RF = RequestFactory()
request = RF.get("/", REMOTE_ADDR="192.168.1.1")
fake_options = {
    "request": request,
    "email": "testuser@fakedomain.com",
    "billing_address": {
        "name": "PayPal User",
        "address1": "Street 1",
        "city": "Mountain View",
        "state": "CA",
        "country": "US",
        "zip": "94043"
        },
    # "shipping_address": {
    #     "name": "PayPal User",
    #     "address1": "Street 1",
    #     "city": "Mountain View",
    #     "state": "CA",
    #     "country": "US",
    #     "zip": "94043"
    #     },
}

class PayPalGatewayTestCase(TestCase):
    def setUp(self):
        self.merchant = get_gateway("pay_pal")
        self.merchant.test_mode = True
        self.credit_card = CreditCard(first_name="Test", last_name="User",
                                      month=10, year=2017, 
                                      number="4500775008976759", 
                                      verification_value="000")

    def testCardSupported(self):
        self.credit_card.number = "5019222222222222"
        self.assertRaises(CardNotSupported, 
                          lambda : self.merchant.purchase(1000, self.credit_card))

    def testCardValidated(self):
        self.merchant.test_mode = False
        self.credit_card.number = "4222222222222123"
        self.assertFalse(self.merchant.validate_card(self.credit_card))

    def testCardType(self):
        self.merchant.validate_card(self.credit_card)
        self.assertEquals(self.credit_card.card_type, Visa)

    def testPurchase(self):
        resp = self.merchant.purchase(1, self.credit_card, 
                                      options = fake_options)
        self.assertEquals(resp["status"], "SUCCESS")
        self.assertNotEquals(resp["response"].correlationid, "0")
        self.assertTrue(isinstance(resp["response"], PayPalNVP)) 

    def testPaymentSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_successful.connect(receive)

        resp = self.merchant.purchase(1, self.credit_card,
                                      options = fake_options)
        self.assertEquals(received_signals, [transaction_was_successful])

    def testPaymentUnSuccessfulSignal(self):
        received_signals = []

        def receive(sender, **kwargs):
            received_signals.append(kwargs.get("signal"))

        transaction_was_unsuccessful.connect(receive)

        resp = self.merchant.purchase(105.02, self.credit_card,
                                      options = fake_options)
        self.assertEquals(received_signals, [transaction_was_unsuccessful])

    def testCreditCardExpired(self):
        resp = self.merchant.purchase(105.02, self.credit_card,
                                      options = fake_options)
        self.assertNotEquals(resp["status"], "SUCCESS")


class PayPalWebsiteStandardsTestCase(TestCase):
    urls = "billing.tests.test_urls"

    def setUp(self):
        self.today = datetime.datetime.today().strftime("%Y-%m-%d")
        self.pws = get_integration("pay_pal")
        self.pws.test_mode = True
        fields = {"cmd": "_xclick",
        'notify_url': 'http://localhost/paypal-ipn-handler/',
        'return_url': 'http://localhost/offsite/paypal/done/',
        'cancel_return': 'http://localhost/offsite/paypal/',
        'amount': 1,
        'item_name': "Test Item",
        'invoice': self.today,}
        self.pws.add_fields(fields)

    def testRenderForm(self):
        tmpl = Template("{% load paypal from paypal_tags %}{% paypal obj %}")
        form = tmpl.render(Context({"obj": self.pws}))
        pregen_form = """<form action="https://www.sandbox.paypal.com/cgi-bin/webscr" method="post"><input type="hidden" name="business" value="probiz_1273571007_biz@uswaretech.com" id="id_business" /><input type="hidden" name="amount" value="1" id="id_amount" /><input type="hidden" name="item_name" value="Test Item" id="id_item_name" /><input type="hidden" name="notify_url" value="http://localhost/paypal-ipn-handler/" id="id_notify_url" /><input type="hidden" name="cancel_return" value="http://localhost/offsite/paypal/" id="id_cancel_return" /><input type="hidden" name="return" value="http://localhost/offsite/paypal/done/" id="id_return_url" /><input type="hidden" name="invoice" value="%(today)s" id="id_invoice" /><input type="hidden" name="cmd" value="_xclick" id="id_cmd" /><input type="hidden" name="charset" value="utf-8" id="id_charset" /><input type="hidden" name="currency_code" value="USD" id="id_currency_code" /><input type="hidden" name="no_shipping" value="1" id="id_no_shipping" /><input type="image" src="https://www.sandbox.paypal.com/en_US/i/btn/btn_buynowCC_LG.gif" border="0" name="submit" alt="Buy it Now" /></form>""" % ( {"today": self.today} )
        self.assertEquals(pregen_form, strip_spaces_between_tags(form).strip())

    def testIPNURLSetup(self):
        self.assertEquals(reverse("paypal-ipn"), "/paypal-ipn-url/")



from django.dispatch import Signal

transaction_started = Signal()

transaction_was_successful = Signal(providing_args=["type", "response"])
transaction_was_unsuccessful = Signal(providing_args=["type", "response"])



from billing import Integration, get_gateway, IntegrationNotConfigured
from django.conf import settings
from django.conf.urls.defaults import patterns, url
from django.views.decorators.csrf import csrf_exempt
from billing.forms.samurai_forms import SamuraiForm


class SamuraiIntegration(Integration):
    display_name = "Samurai"

    def __init__(self):
        super(SamuraiIntegration, self).__init__()
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("samurai"):
            raise IntegrationNotConfigured("The '%s' integration is not correctly "
                                       "configured." % self.display_name)
        samurai_settings = merchant_settings["samurai"]
        self.merchant_key = samurai_settings['MERCHANT_KEY']
        self.gateway = get_gateway("samurai")

    def generate_form(self):
        initial_data = self.fields
        form = SamuraiForm(initial=initial_data)
        return form
        
    @csrf_exempt
    def transaction(self, request):
        raise NotImplementedError

    def get_urls(self):
        urlpatterns = patterns('',
           url('^samurai_token/$', self.transaction, name="samurai_transaction")
        )
        return urlpatterns



from billing import Gateway, GatewayNotConfigured
from billing.signals import transaction_was_successful, transaction_was_unsuccessful
from billing.utils.credit_card import InvalidCard, Visa, MasterCard, \
     AmericanExpress, Discover, CreditCard
import stripe
from django.conf import settings


class StripeGateway(Gateway):
    supported_cardtypes = [Visa, MasterCard, AmericanExpress, Discover]
    supported_countries = ['US']
    default_currency = "USD"
    homepage_url = "https://stripe.com/"
    display_name = "Stripe"

    def __init__(self):
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("stripe"):
            raise GatewayNotConfigured("The '%s' gateway is not correctly "
                                       "configured." % self.display_name)
        stripe_settings = merchant_settings["stripe"]
        stripe.api_key = stripe_settings['API_KEY']
        self.stripe = stripe

    def purchase(self, amount, credit_card, options=None):
        card = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            card = {
                'number': credit_card.number,
                'exp_month': credit_card.month,
                'exp_year': credit_card.year,
                'cvc': credit_card.verification_value
                }
        try:
            response = self.stripe.Charge.create(
                amount=amount * 100,
                currency=self.default_currency.lower(),
                card=card)
        except self.stripe.CardError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="purchase",
                                              response=response)
            return {'status': 'FAILURE', 'response': error}
        transaction_was_successful.send(sender=self, 
                                        type="purchase",
                                        response=response)
        return {'status': 'SUCCESS', 'response': response}

    def store(self, credit_card, options=None):
        card = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            card = {
                'number': credit_card.number,
                'exp_month': credit_card.month,
                'exp_year': credit_card.year,
                'cvc': credit_card.verification_value
                }
        try:
            customer = self.stripe.Customer.create(card=card)
        except (self.stripe.CardError, self.stripe.InvalidRequestError), error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="store",
                                              response=error)
            return {'status': 'FAILURE', 'response': error}
        transaction_was_successful.send(sender=self, 
                                        type="store",
                                        response=customer)
        return {'status': 'SUCCESS', 'response': customer}

    def recurring(self, credit_card, options=None):
        card = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            card = {
                'number': credit_card.number,
                'exp_month': credit_card.month,
                'exp_year': credit_card.year,
                'cvc': credit_card.verification_value
                }
        try:
            plan_id = options['plan_id']
            self.stripe.Plan.retrieve(options['plan_id'])
            try:
                response = self.stripe.Customer.create(
                    card=card,
                    plan=plan_id
                )
                transaction_was_successful.send(sender=self, 
                                                type="recurring",
                                                response=response)
                return {"status": "SUCCESS", "response": response}
            except self.stripe.CardError, error:
                transaction_was_unsuccessful.send(sender=self, 
                                                  type="recurring",
                                                  response=error)
                return {"status": "FAILURE", "response": error}
        except self.stripe.InvalidRequestError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="recurring",
                                              response=error)
            return {"status": "FAILURE", "response": error}
        except TypeError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="recurring",
                                              response=error)
            return {"status": "FAILURE", "response": "Missing Plan Id"}

    def unstore(self, identification, options=None):
        try:
            customer = self.stripe.Customer.retrieve(identification)
            response = customer.delete()
            transaction_was_successful.send(sender=self, 
                                              type="unstore",
                                              response=response)
            return {"status": "SUCCESS", "response": response}
        except self.stripe.InvalidRequestError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="unstore",
                                              response=error)
            return {"status": "FAILURE", "response": error}

    def credit(self, identification, money=None, options=None):
        try:
            charge = self.stripe.Charge.retrieve(identification)
            response = charge.refund(amount=money)
            transaction_was_successful.send(sender=self, 
                                            type="credit",
                                            response=response)
            return {"status": "SUCCESS", "response": response}
        except self.stripe.InvalidRequestError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="credit",
                                              response=error)
            return {"status": "FAILURE", "error": error}

    def authorize(self, money, credit_card, options=None):
        card = credit_card
        if isinstance(credit_card, CreditCard):
            if not self.validate_card(credit_card):
                raise InvalidCard("Invalid Card")
            card = {
                'number': credit_card.number,
                'exp_month': credit_card.month,
                'exp_year': credit_card.year,
                'cvc': credit_card.verification_value
                }
        try:
            token = self.stripe.Token.create(
                card=card,
                amount=money * 100
            )
            transaction_was_successful.send(sender=self, 
                                            type="authorize",
                                            response=token)
            return {'status': "SUCCESS", "response": token}
        except self.stripe.InvalidRequestError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="authorize",
                                              response=error)
            return {"status": "FAILURE", "response": error}

    def capture(self, money, authorization, options=None):
        try:
            response = self.stripe.Charge.create(
                amount=money * 100,
                card=authorization,
                currency=self.default_currency.lower()
            )
            transaction_was_successful.send(sender=self, 
                                            type="capture",
                                            response=response)
            return {'status': "SUCCESS", "response": response}
        except self.stripe.InvalidRequestError, error:
            transaction_was_unsuccessful.send(sender=self, 
                                              type="capture",
                                              response=error)
            return {"status": "FAILURE", "response": error}



from billing import Integration, get_gateway, IntegrationNotConfigured
from django.conf import settings
from django.conf.urls.defaults import patterns, url
from billing.forms.stripe_forms  import StripeForm


class StripeIntegration(Integration):
    display_name = "Stripe"

    def __init__(self):
        super(StripeIntegration, self).__init__()
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("stripe"):
            raise IntegrationNotConfigured("The '%s' integration is not correctly "
                                       "configured." % self.display_name)
        stripe_settings = merchant_settings["stripe"]
        self.gateway = get_gateway("stripe")
        self.publishable_key = stripe_settings['PUBLISHABLE_KEY']

    def generate_form(self):
        initial_data = self.fields
        form = StripeForm(initial=initial_data)
        return form

    def transaction(self, request):
        # Subclasses must override this
        raise NotImplementedError

    def get_urls(self):
        urlpatterns = patterns('',
           url('^stripe_token/$', self.transaction, name="stripe_transaction")
        )
        return urlpatterns



import unittest
from base_tests import *
from authorize_net_tests import *
from pay_pal_tests import *
from eway_tests import *
from world_pay_tests import *
from google_checkout_tests import *
from amazon_fps_tests import *
from braintree_payments_tests import *
from braintree_payments_tr_tests import *
from stripe_tests import *
from samurai_tests import *

if __name__ == "__main__":
    unittest.main()
